{ config, pkgs, lib, ... }:

let
  user = "{{ system_user_username }}";
  password = "{{ system_user_password }}";
  hostname = "{{ hostname }}";
  userID = "1000"; # UserID for the unifi-controller coontainer

in {
  boot = {
    kernelPackages = pkgs.linuxKernel.packages.linux_rpi4;
    initrd = {
      availableKernelModules = [ "nfs" ];
      supportedFilesystems = [ "nfs" ];
    };
    loader = {
      grub.enable = false;
      generic-extlinux-compatible.enable = true;
    };
  };

  environment.systemPackages = with pkgs; [
    vim # needed for manual intervention
    python3 # needed for ansible
    pkgs.podman # needed to run unifi-controller
    nfs-utils # needed to mount nfs-share
  ];

  fileSystems = {
    "/" = {
      device = "/dev/disk/by-label/NIXOS_SD";
      fsType = "ext4";
      options = [ "noatime" ];
    };
  };

  networking = {
    hostName = hostname;
    wireless = {
      enable = false;
    };
    enableIPv6 = false;
    firewall = {
      enable = true;
      allowedTCPPorts = [ 8443 8080 8843 8880 6789 ];
      allowedUDPPorts = [ 3478 10001 1900 5514 ];
    };
  };

  security.sudo = {
    enable = true;
    extraRules = [{
      # commands = [
      #   {
      #     command = "/run/current-system/sw/bin/nixos-rebuild test";
      #     options = [ "NOPASSWD" ];
      #   }
      #   {
      #     command = "/run/current-system/sw/bin/nixos-rebuild switch";
      #     options = [ "NOPASSWD" ];
      #   }
      #   {
      #     command = "/run/current-system/sw/bin/nixos-rebuild boot";
      #     options = [ "NOPASSWD" ];
      #   }
      # ];
      # groups = ["wheel"];
      # Allow unifi-controller user to sudo without password because ansible
      users = [ user ];
      commands = [{
        command = "ALL";
        options = [ "NOPASSWD" ];
      }];
    }];
  };

  services = {
    rpcbind.enable = true; # needed for NFS
    nfs.server.enable = true; # needed for NFS mount O.o
    openssh = {
      enable = true;
      settings.PasswordAuthentication = false;
      settings.KbdInteractiveAuthentication = false;
    };
  };

  system = {
    stateVersion = "23.05";
  };

  systemd = {
    mounts = [{
      type = "nfs";
      mountConfig = {
        Options = "noatime";
      };
      what = "{{ nfs_server_ip }}:{{ nfs_server_share }}";
      where = "/nfs";
    }];

    automounts = [{
      wantedBy = [ "multi-user.target" "podman-unifi-controller.service" ];
      automountConfig = {
        TimeoutIdleSec = "600";
      };
      where = "/nfs";
    }];
  };

  virtualisation = {
    podman = {
      enable = true;
      dockerSocket.enable = true;
    };

    oci-containers.containers = {
      unifi-controller = {
        image = "{{ unifi_controller_image_name }}:{{ unifi_controller_image_tag }}";
        autoStart = true;
        extraOptions = [ "--network=host" ];
        volumes = [ "/nfs/config:/config" ];
        environment = {
            PUID = userID;
            PGID = userID;
            TZ = "Europe/Madrid";
            MEM_LIMIT = "1024";
            MEM_STARTUP = "1024";
        };
        ports = [
            "8443:8443"
            "3478:3478/udp"
            "10001:10001/udp"
            "8080:8080"
            "1900:1900/udp"
            "8843:8843"
            "8880:8880"
            "6789:6789"
            "5514:5514/udp"
        ];
      };
    };
  };

  users = {
    mutableUsers = false;
    users."${user}" = {
      uid = 1001;
      isNormalUser = true;
      password = password;
      extraGroups = [ "wheel" ];
      openssh.authorizedKeys.keys = [
        "{{ system_user_ssh_public_key }}"
      ];
    };
  };

  hardware.enableRedistributableFirmware = true;
}
